@@ -32,6 +32,7 @@ Main code for controltower.
 """
 
 import copy
+import itertools
 import json
 import logging
 import time
@@ -116,11 +117,11 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         def wrap(*args, **kwargs):
             """Inner wrapper decorator."""
             logger = logging.getLogger(f'{LOGGER_BASENAME}.validation_decorator')
-            contol_tower_instance = args[0]
+            control_tower_instance = args[0]
             logger.debug('Decorating method: %s', method)
-            if not contol_tower_instance.is_deployed:
+            if not control_tower_instance.is_deployed:
                 raise ControlTowerNotDeployed
-            if contol_tower_instance.busy:
+            if control_tower_instance.busy:
                 raise ControlTowerBusy
             return method(*args, **kwargs)  # pylint: disable=not-callable
 
@@ -138,7 +139,6 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         self._account_factory_ = None
         self.settling_time = settling_time
         self._root_ou = None
-        self._update_data_ = None
         self._core_accounts = None
 
     @property
@@ -332,10 +332,8 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
 
     @property
     def _update_data(self):
-        if self._update_data_ is None:
-            self._update_data_ = next(self._get_paginated_results(content_payload={},
-                                                                  target='getAvailableUpdates'))
-        return self._update_data_
+        return next(self._get_paginated_results(content_payload={},
+                                                target='getAvailableUpdates'), {})
 
     @property
     @validate_availability
@@ -526,9 +524,12 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
             organizational_units (OrganizationsOU): A list of organizational units objects under Organizations.
 
         """
-        response = self.organizations.list_organizational_units_for_parent(ParentId=self.root_ou.id)
+        paginator = self.organizations.get_paginator('list_organizational_units_for_parent')
+        pages = paginator.paginate(ParentId=self.root_ou.id)
+        response = list(itertools.chain.from_iterable([page['OrganizationalUnits'] for page in pages]))
+
         return [OrganizationsOU(data)
-                for data in response.get('OrganizationalUnits', [])]
+                for data in response]
 
     @validate_availability
     def get_organizations_ou_by_name(self, name):
@@ -718,6 +719,8 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
             if not self.create_organizational_unit(organizational_unit):
                 self.logger.error('Unable to create the organizational unit!')
                 return False
+        while self.busy:
+            time.sleep(1)
         arguments = {'ProductId': self._account_factory.product_id,
                      'ProvisionedProductName': product_name,
                      'ProvisioningArtifactId': self._active_artifact.get('Id'),
@@ -734,8 +737,6 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
                                                 {'Key': 'ManagedOrganizationalUnit',
                                                  'Value': organizational_unit}]}
         try:
-            while self.busy:
-                time.sleep(1)
             response = self.service_catalog.provision_product(**arguments)
         except botocore.exceptions.ClientError as err:
             if CREATING_ACCOUNT_ERROR_MESSAGE in err.response['Error']['Message']:
@@ -780,7 +781,7 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
 
     @validate_availability
     def update(self):
-        """Updates the control tower to the latest version.
+        """Updates the control tower to the next available version.
 
         Returns:
             bool: True on success, False on failure.
@@ -796,10 +797,7 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         security_account = next((account for account in self.core_accounts if account.label == 'SECURITY'), None)
         if not security_account:
             raise ServiceCallFailed('Could not retrieve security account to get the email.')
-        payload = self._get_api_payload(content_string={'HomeRegion': self.region,
-                                                        'LogAccountEmail': log_account.email,
-                                                        'SecurityAccountEmail': security_account.email},
-                                        target='setupLandingZone')
+        payload = self._get_update_payload(log_account.email, security_account.email)
         self.logger.debug('Trying to update the landing zone with payload "%s"', payload)
         response = self.session.post(self.url, json=payload)
         if not response.ok:
@@ -807,8 +805,31 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
                               response.status_code, response.text)
             return False
         self.logger.debug('Successfully started updating landing zone')
+        # Making sure that eventual consistency is not a problem here,
+        # we wait for control tower to be aware of the service catalog process
+        while not self.busy:
+            time.sleep(1)
         return True
 
+    def _get_update_payload(self, log_account_email, security_account_email):
+        """Updates the control tower up to 2.6 version.
+
+        Returns:
+            bool: True on success, False on failure.
+
+        """
+        content = {'HomeRegion': self.region,
+                   'LogAccountEmail': log_account_email,
+                   'SecurityAccountEmail': security_account_email}
+        if self.landing_zone_version == '2.6':
+            region_list = [{"Region": region,
+                            "RegionConfigurationStatus": "ENABLED" if region in self.governed_regions else "DISABLED"}
+                           for region in self.get_available_regions()]
+            content.update({'SetupLandingZoneActionType': 'UPDATE',
+                            'RegionConfigurationList': region_list})
+        return self._get_api_payload(content_string=content,
+                                     target='setupLandingZone')
+
     @property
     def busy(self):
         """Busy."""
@@ -837,6 +858,18 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         """Region metadata list."""
         return self._get_status().get('RegionMetadataList')
 
+    @property
+    def governed_regions(self):
+        """Governed regions."""
+        return [region.get('Region')
+                for region in self.region_metadata_list if region.get('RegionStatus') == 'GOVERNED']
+
+    @property
+    def not_governed_regions(self):
+        """Not governed regions."""
+        return [region.get('Region')
+                for region in self.region_metadata_list if region.get('RegionStatus') == 'NOT_GOVERNED']
+
     def _get_status(self):
         payload = self._get_api_payload(content_string={},
                                         target='getLandingZoneStatus')
@@ -959,12 +992,17 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
                    'region': 'us-east-1'}
         self.logger.debug('Trying to system role with payload "%s"', payload)
         response = self.session.post(self._iam_admin_url, json=payload)
-        if all([not response.ok,
-                response.status_code == 409,
-                response.json().get('Error', {}).get('Code') == 'EntityAlreadyExists]']):
-            self.logger.error('Entity already exists, response status "%s" and response text "%s"',
+        try:
+            if all([not response.ok,
+                    response.status_code == 409,
+                    response.json().get('Error', {}).get('Code') == 'EntityAlreadyExists]']):
+                self.logger.error('Entity already exists, response status "%s" and response text "%s"',
+                                  response.status_code, response.text)
+                return True
+        except ValueError:
+            self.logger.error('Error on request, response status "%s" and response text "%s"',
                               response.status_code, response.text)
-            return True
+            return False
         if not response.ok:
             self.logger.error('Entity already exists, response status "%s" and response text "%s"',
                               response.status_code, response.text)
@@ -1082,6 +1120,10 @@ class ControlTower(LoggerMixin):  # pylint: disable=too-many-instance-attributes
             self.logger.error('Failed to deploy control tower, retries were spent.. Maybe try again later?')
             return False
         self.logger.debug('Successfully started deploying control tower.')
+        # Making sure that eventual consistency is not a problem here,
+        # we wait for control tower to be aware of the service catalog process
+        while not self.busy:
+            time.sleep(1)
         return True
 
     def decommission(self):
